Set-StrictMode -Version Latest

# Errors don't normally terminate script execution
$ErrorActionPreference = "Stop"

# Load the assembly from the path adjacent to this module.
$path = Split-Path $script:MyInvocation.MyCommand.Path
[Reflection.Assembly]::LoadFrom("$path\Microsoft.IdentityModel.Clients.ActiveDirectory.dll") | Out-Null

function Export-PlannerUserContent
{
    param(
        [Parameter(Mandatory=$true)]
        [ValidateNotNull()]
        [string] $UserAadIdOrPrincipalName,

        [Parameter(Mandatory=$true)]
        [ValidateNotNull()]
        [string] $ExportDirectory,

        [ValidateNotNull()]
        [string] $HostName,

        [ValidateNotNull()]
        [Microsoft.IdentityModel.Clients.ActiveDirectory.AuthenticationResult, Microsoft.IdentityModel.Clients.ActiveDirectory, Version=5.2.8.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35] $Authentication,

        [guid] $CorrelationId
    )

    if(!$HostName)
    {
        $HostName = "tasks.office.com"
    }

    if(!$Authentication)
    {
        $Authentication = Connect-AAD -HostName $HostName
    }

    if(!$CorrelationId)
    {
        $CorrelationId = [guid]::NewGuid()
    }

    if($UserAadIdOrPrincipalName -match '[\/\\\:\*\"\<\>\|]')
    {
        throw "The id or name contains the unsupported character $($matches[0])"
    }

    [string] $userExportDirectory = Join-Path $ExportDirectory $UserAadIdOrPrincipalName

    if(!(Test-Path -PathType Container $ExportDirectory))
    {
        throw "The directory you chose, `"$ExportDirectory`", does not exist. To continue, create it or choose another directory."
    }

    if(Test-Path $userExportDirectory -PathType Leaf)
    {
        throw "There is already a file named `"$userExportDirectory`". Remove it to continue."
    }

    if(Test-Path $userExportDirectory -PathType Container)
    {
        [int] $childItems = (Get-ChildItem).Count
        if($childItems -gt 0)
        {
            throw "There is already a folder named `"$userExportDirectory`". Remove it to continue."
        }        
    }

    New-Item -ItemType Directory $userExportDirectory | out-null

    [uri] $sessionRedirectUri = Start-PlannerUserContentExport -HostName $HostName -Authentication $Authentication -UserAadIdOrPrincipalName $UserAadIdOrPrincipalName -CorrelationId $CorrelationId

    [uri[]] $uris = $null

    $consecutiveErrors = 0
    while(!$uris)
    {
        sleep 10

        try
        {
            $uris = Get-PlannerUserContentExport -SessionRedirectUri $sessionRedirectUri -Authentication $Authentication -CorrelationId $CorrelationId
            $consecutiveErrors = 0
        }
        catch
        {
            ThrowExceptionIfTerminal -Exception $_.Exception -CorrelationId $CorrelationId

            $consecutiveErrors++
            if($consecutiveErrors -gt 3)
            {            
                ThrowGenericExportError -CorrelationId $CorrelationId
            }
        }
    }    

    foreach($uri in $uris)
    {
        $success = $false
        $consecutiveErrors = 0
        while(!$success)
        {
            try
            {
                Copy-FileToExportDirectory -UriToDownload $uri -UserExportDirectory $userExportDirectory
                $success = $true
                $consecutiveErrors = 0
            }
            catch
            {
                ThrowExceptionIfTerminal -Exception $_.Exception -CorrelationId $CorrelationId

                $consecutiveErrors++
                if($consecutiveErrors -gt 3)
                {
                    ThrowGenericExportError -CorrelationId $CorrelationId
                }
                else
                {
                    sleep 10
                }
            }
        }        
    }

    return $userExportDirectory
}

function ThrowExceptionIfTerminal
{
    param(
        [Parameter(Mandatory=$true)]
        [ValidateNotNull()]
        [exception] $Exception,

        [Parameter(Mandatory=$true)]
        [guid] $CorrelationId
    )
   
    $terminal = $false;
    $webException = $Exception -as [System.Net.WebException]
    if($webException)
    {
        $response = $webException.Response -as [System.Net.HttpWebResponse]
        if($response -eq $null)
        {
            $terminal = $true;
        }
        else
        {
            $statusCode = $response.StatusCode
            if(($statusCode -eq [System.Net.HttpStatusCode]::Unauthorized) `
                -or ($statusCode -eq [System.Net.HttpStatusCode]::NotFound) `
                -or ($statusCode -eq [System.Net.HttpStatusCode]::BadRequest) `
                -or ($statusCode -eq [System.Net.HttpStatusCode]::Forbidden) `
                -or (([int]$statusCode) -eq 429) ` #TooManyRequests
                )
            {
                $terminal = $true;
            }
        }
    }

    if($terminal)
    {
        $message = "We couldn't start your export. Error message: $($Exception.Message) Support information: CorrelationId $CorrelationId"
        $exceptionToThrow = new-object system.management.automation.runtimeexception $message, $Exception
        throw $exceptionToThrow
    }
}

function ThrowGenericExportError
{
    param(
        [Parameter(Mandatory=$true)]
        [guid] $CorrelationId
    )

    throw "We couldn't start your export. Please try again in a few minutes. Support information: CorrelationId $CorrelationId"
}

function Copy-FileToExportDirectory
{
    param(
        [Parameter(Mandatory=$true)]
        [ValidateNotNull()]
        [uri] $UriToDownload,

        [Parameter(Mandatory=$true)]
        [ValidateNotNull()]
        [string] $UserExportDirectory
    )
   
    $fileName = Get-FileName -Uri $UriToDownload
    $targetFilePath = Join-Path $UserExportDirectory $fileName

    if(Test-Path $targetFilePath)
    {
        throw "Could not download to $targetFilePath. The file already exists."
    }

    Invoke-WebRequest -Uri $UriToDownload -OutFile $targetFilePath
}

function Get-FileName
{
    param(
        [Parameter(Mandatory=$true)]
        [ValidateNotNull()]
        [uri] $Uri
    )

    # Exclude query string and trim trailing slashes.
    $lastSegment = $Uri.Segments[-1].Trim("/")

    # If the Uri is just the domain, the only segment is the slash.
    if($lastSegment -eq "")
    {
        throw "The URI does not contain a file name."
    }

    return $lastSegment
}

function Start-PlannerUserContentExport
{
    param(
        [Parameter(Mandatory=$true)]
        [ValidateNotNull()]
        [string] $HostName,

        [Parameter(Mandatory=$true)]
        [ValidateNotNull()]
        [Microsoft.IdentityModel.Clients.ActiveDirectory.AuthenticationResult, Microsoft.IdentityModel.Clients.ActiveDirectory, Version=5.2.8.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35] $Authentication,

        [Parameter(Mandatory=$true)]
        [ValidateNotNull()]
        [string] $UserAadIdOrPrincipalName,

        [Parameter(Mandatory=$true)]
        [guid] $CorrelationId
    )

    $uri = "https://" + $HostName + "/taskApi/UserContentExport/UserContentExport"
    $queryParameters = $UserAadIdOrPrincipalName

    try
    {
        [Microsoft.PowerShell.Commands.WebResponseObject] $response = Invoke-ContentExportMethod -Uri $uri -HttpVerb Post -QueryParameters $queryParameters -Authentication $Authentication -CorrelationId $CorrelationId
        $redirect = $response.Headers["Location"]
        return [uri] $redirect
    }
    catch
    {
        ThrowExceptionIfTerminal -Exception $_.Exception -CorrelationId $CorrelationId
        
        ThrowGenericExportError -CorrelationId $CorrelationId
    }
}

function Get-PlannerUserContentExport
{
    param(
        [Parameter(Mandatory=$true)]
        [ValidateNotNull()]
        [Microsoft.IdentityModel.Clients.ActiveDirectory.AuthenticationResult, Microsoft.IdentityModel.Clients.ActiveDirectory, Version=5.2.8.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35] $Authentication,

        [Parameter(Mandatory=$true)]
        [ValidateNotNull()]
        [uri] $SessionRedirectUri,

        [Parameter(Mandatory=$true)]
        [guid] $CorrelationId
    )

    [Microsoft.PowerShell.Commands.WebResponseObject] $response = Invoke-ContentExportMethod -HttpVerb Get -Uri $SessionRedirectUri -Authentication $Authentication -CorrelationId $CorrelationId

    if($response.StatusCode -eq [System.Net.HttpStatusCode]::OK)
    {
        [uri[]] $uris = ConvertFrom-Json $response.Content
        return $uris
    }
    else
    {
        return $null
    }
}

function Invoke-ContentExportMethod
{
    param(
        [Parameter(Mandatory=$true)]
        [ValidateNotNull()]
        [uri]$Uri,

        [Parameter(Mandatory=$true)]
        [ValidateNotNull()]
        [string]$HttpVerb,

        [ValidateNotNull()]
        $QueryParameters,

        [Parameter(Mandatory=$true)]
        [ValidateNotNull()]
        [Microsoft.IdentityModel.Clients.ActiveDirectory.AuthenticationResult, Microsoft.IdentityModel.Clients.ActiveDirectory, Version=5.2.8.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35] $Authentication,

        [Parameter(Mandatory=$true)]
        [guid] $CorrelationId
    )
    
    [string] $body = $null

    if($QueryParameters)
    {    
        if($HttpVerb -eq "GET")
        {
            if ($queryParameters -isnot [hashtable])
            {
                throw "Get method query parameters must be a hashtable"
            }

            $combinedQueryString = Get-QueryString $QueryParameters
            $Uri = "$($Uri)?$combinedQueryString"
        }
        else
        {
            $body = $QueryParameters | ConvertTo-Json
        }
    }

    $accessTokenType = $Authentication.AccessTokenType
    $accessToken = $Authentication.AccessToken
    $accessHeader = $accessTokenType + " " + $accessToken
    $contentType = "application/json"
    $headers = @{"Accept"="application/json"; "Authorization"=$accessHeader; "Accept-Charset"="UTF-8"; "X-CorrelationId"="$CorrelationId"}

    if($body)
    {    
        $response = Invoke-WebRequest -ContentType $contentType -Headers $headers -Method $HttpVerb -Body $body $Uri
    }
    else
    {
        $response = Invoke-WebRequest -ContentType $contentType -Headers $headers -Method $HttpVerb $Uri
    }
    
    return $response
}

function Get-QueryString
{
    param(
        [ValidateNotNull()]
        [hashtable] $QueryParameters
    )

    $queryStrings = $QueryParameters.GetEnumerator() | %{"$($_.Name)=$($_.Value)"}
    $combinedQueryString = [string]::Join("&", $queryStrings)

    return $combinedQueryString
}

function Connect-AAD
{
    param(
        [string] $HostName
    )

    if ($HostName -eq "tasks.office365.us")
    {
        $authUrl = "https://login.microsoftonline.us/common"
        $resource = "https://tasks.office365.us"
    }
    elseif ($HostName -eq "tasks.osi.apps.mil")
    {
        $authUrl = "https://login.microsoftonline.us/common"
        $resource = "https://tasks.osi.apps.mil"
    }
    else
    {
        # Prod environment
        $authUrl = "https://login.microsoftonline.com/common"
        $resource = "https://tasks.office.com"
    }

    $clientId = "9ac8c0b3-2c30-497c-b4bc-cadfe9bd6eed"
    
    $redirectUri = "https://login.microsoftonline.com/common/oauth2/nativeclient"
    $platformParameters = New-Object "Microsoft.IdentityModel.Clients.ActiveDirectory.PlatformParameters, Microsoft.IdentityModel.Clients.ActiveDirectory, Version=5.2.8.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35" -ArgumentList "Auto"
    
    $authentiationContext = New-Object "Microsoft.IdentityModel.Clients.ActiveDirectory.AuthenticationContext, Microsoft.IdentityModel.Clients.ActiveDirectory, Version=5.2.8.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35" -ArgumentList $authUrl, $False
    
    $authenticationResult = $authentiationContext.AcquireTokenAsync($resource, $clientId, $redirectUri, $platformParameters).Result
    return $authenticationResult
}

function Get-PlannerUserPolicy
{
<#
.Synopsis
Retrieves a user's policy for Microsoft Planner.
.Description
This cmdlet allows tenant administrators to retrieve user policy set by the tenant administrator for specific user
.Parameter HostName
The host name of the UserPolicy API for the Planner instance to retrieve. Default tasks.office.com
.Parameter Authentication
Authentication of a user with tenant-level administrator privileges.
.Parameter UserAadIdOrPrincipalName
AAD id or UPN for user
.example
PS> Get-PlannerUserPolicy -UserAadIdOrPrincipalName aadIdOfUser | fl
@odata.context                   : https://tasks.office.com/taskApi/tenantAdminSettings/$metadata#UserPolicy/$entity
id                               : aadIdOfUser
blockDeleteTasksNotCreatedBySelf : False
#>
    param(
        [Parameter(Mandatory=$true)]
        [ValidateNotNullOrEmpty()]
        [System.String]$UserAadIdOrPrincipalName,

        [ValidateNotNull()]
        [Microsoft.IdentityModel.Clients.ActiveDirectory.AuthenticationResult, Microsoft.IdentityModel.Clients.ActiveDirectory, Version=5.2.8.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35] $Authentication,

        [ValidateNotNull()]
        [string] $HostName
        )
    
    if(!$HostName)
    {
        $HostName = "tasks.office.com"
    }

    if(!$Authentication)
    {
        $Authentication = Connect-AAD -HostName $HostName
    }

    $AccessToken = $Authentication.AccessTokenType.ToString() + ' ' +$Authentication.AccessToken    
    $UserUri = "https://" + $HostName + "/taskAPI/tenantAdminSettings/UserPolicy('" + $UserAadIdOrPrincipalName + "')";
    return Invoke-RestMethod -ContentType "application/json" -Headers @{"Accept"="application/json"; "Authorization"=$AccessToken; "Accept-Charset"="UTF-8"; "OData-Version"="4.0;NetFx"; "OData-MaxVersion"="4.0;NetFx"} -Method GET $UserUri
}

function Set-PlannerUserPolicy
{
<#
.Synopsis
Sets a user's policy for Microsoft Planner.
.Description
This cmdlet allows tenant administrators to set user policy for a specific user
.Parameter HostName
The host name of the UserPolicy API for the Planner instance to retrieve. Default: tasks.office.com
.Parameter Authentication
Authentication result of a user with tenant-level administrator privileges.
.Parameter UserAadIdOrPrincipalName
AAD id or UPN for user
.Parameter BlockDeleteTasksNotCreatedBySelf
Whether to block deleting tasks not created by user
.example
PS> Set-PlannerUserPolicy -UserId aadIdOfUser -BlockDeleteTasksNotCreatedBySelf $true
.example
PS> Set-PlannerUserPolicy -UserId aadIdOfUser -BlockDeleteTasksNotCreatedBySelf $false
#>
    param(
        [Parameter(Mandatory=$true)]
        [ValidateNotNullOrEmpty()]
        [System.String]$UserAadIdOrPrincipalName,

        [ValidateNotNull()]
        [Microsoft.IdentityModel.Clients.ActiveDirectory.AuthenticationResult, Microsoft.IdentityModel.Clients.ActiveDirectory, Version=5.2.8.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35] $Authentication,

        [Parameter(Mandatory=$true)]
        [System.Boolean]$BlockDeleteTasksNotCreatedBySelf,

        [ValidateNotNull()]
        [System.String] $HostName
        )
    
    if(!$HostName)
    {
        $HostName = "tasks.office.com"
    }

    if(!$Authentication)
    {
        $Authentication = Connect-AAD -HostName $HostName
    }

    $AccessToken = $Authentication.AccessTokenType.ToString() + ' ' +$Authentication.AccessToken    
    $Uri = "https://" + $HostName + "/taskAPI/tenantAdminSettings/UserPolicy('" + $UserAadIdOrPrincipalName + "')";

    $userPolicy = @{};
    $userPolicy.Add("blockDeleteTasksNotCreatedBySelf", $BlockDeleteTasksNotCreatedBySelf);
    $userPolicy.Add("id", $UserAadIdOrPrincipalName);
    $requestBody = $userPolicy | ConvertTo-Json
    
    Invoke-RestMethod -ContentType "application/json" -Headers @{"Accept"="application/json"; "Authorization"=$AccessToken; "Accept-Charset"="UTF-8"; "OData-Version"="4.0;NetFx"; "OData-MaxVersion"="4.0;NetFx"} -Method PUT -Body $requestBody $Uri
}

function Add-PlannerRosterMember
{
    <#
    .Synopsis
    Adds a member to the specified Roster in Microsoft Planner

    .Description
    This cmdlet allows tenant administrators to add members to Rosters in Microsoft Planner.

    .Parameter RosterId
    The id of the Roster to add a member to

    .Parameter UserAadIdOrPrincipalName
    AAD id or UPN if the user to add

    .Parameter Authentication
    Optional. An AuthenticationResult obtained from the Microsoft Identity library. When this parameter is not set, the user will be prompted for credentials.

    .Parameter HostName
    Optional. The host name of the Microsoft Planner service to target. Default is tasks.office.com.

    .example
    PS> Add-PlannerRosterMember -RosterId "sample" -UserAadIdOrPrincipalName "user@contoso.com"
    
    <UserAADId>
    #>
    param(
        [Parameter(Mandatory=$true)]
        [ValidateNotNullOrEmpty()]
        [System.String]$RosterId,

        [Parameter(Mandatory=$true)]
        [ValidateNotNullOrEmpty()]
        [System.String]$UserAadIdOrPrincipalName,

        [ValidateNotNull()]
        [Microsoft.IdentityModel.Clients.ActiveDirectory.AuthenticationResult, Microsoft.IdentityModel.Clients.ActiveDirectory, Version=5.2.8.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35] $Authentication,

        [ValidateNotNull()]
        [string] $HostName
        )
    
    if(!$HostName)
    {
        $HostName = "tasks.office.com"
    }

    if(!$Authentication)
    {
        $Authentication = Connect-AAD -HostName $HostName
    }

    $AccessToken = $Authentication.AccessTokenType.ToString() + ' ' + $Authentication.AccessToken
    $Uri = "https://" + $HostName + "/taskAPI/v3.0/rosters('" + $RosterId + "')/members";
    $Body = @{"userId" = $UserAadIdOrPrincipalName } | ConvertTo-Json
    $Result = Invoke-RestMethod -ContentType "application/json" -Headers @{"Accept"="application/json"; "Authorization"=$AccessToken; "Accept-Charset"="UTF-8"} -Method POST $Uri -Body $Body
    return $Result.userId
}

function Remove-PlannerRosterMember
{
    <#
    .Synopsis
    Removes a member from the specified Roster in Microsoft Planner

    .Description
    This cmdlet allows tenant administrators to remove members from Rosters in Microsoft Planner.

    .Parameter RosterId
    The id of the Roster to remove a member from

    .Parameter UserAadIdOrPrincipalName
    AAD id or UPN if the user to remove

    .Parameter Authentication
    Optional. An AuthenticationResult obtained from the Microsoft Identity library. When this parameter is not set, the user will be prompted for credentials.

    .Parameter HostName
    Optional. The host name of the Microsoft Planner service to target. Default is tasks.office.com.

    .example
    PS> Remove-PlannerRosterMember -RosterId "sample" -UserAadIdOrPrincipalName "user@contoso.com"
    #>
    param(
        [Parameter(Mandatory=$true)]
        [ValidateNotNullOrEmpty()]
        [System.String]$RosterId,

        [Parameter(Mandatory=$true)]
        [ValidateNotNullOrEmpty()]
        [System.String]$UserAadIdOrPrincipalName,

        [ValidateNotNull()]
        [Microsoft.IdentityModel.Clients.ActiveDirectory.AuthenticationResult, Microsoft.IdentityModel.Clients.ActiveDirectory, Version=5.2.8.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35] $Authentication,

        [ValidateNotNull()]
        [string] $HostName
        )
    
    if(!$HostName)
    {
        $HostName = "tasks.office.com"
    }

    if(!$Authentication)
    {
        $Authentication = Connect-AAD -HostName $HostName
    }

    $AccessToken = $Authentication.AccessTokenType.ToString() + ' ' + $Authentication.AccessToken    
    $Uri = "https://" + $HostName + "/taskAPI/v3.0/rosters('" + $RosterId + "')/members('" + $UserAadIdOrPrincipalName + "')";

    Invoke-RestMethod -ContentType "application/json" -Headers @{"Accept"="application/json"; "Authorization"=$AccessToken; "Accept-Charset"="UTF-8"} -Method DELETE $Uri
}

function Get-PlannerRosterMember
{
    <#
    .Synopsis
    Queries members of a Roster in Microsoft Planner

    .Description
    This cmdlet allows tenant administrators to query the members of a Roster in Microsoft Planner.

    .Parameter RosterId
    The id of the Roster query members for

    .Parameter Authentication
    Optional. An AuthenticationResult obtained from the Microsoft Identity library. When this parameter is not set, the user will be prompted for credentials.

    .Parameter HostName
    Optional. The host name of the Microsoft Planner service to target. Default is tasks.office.com.

    .example
    PS> Get-PlannerRosterMember -RosterId "sample"

    <UserAADId 1>
    <UserAADId 2>
    #>
    param(
        [Parameter(Mandatory=$true)]
        [ValidateNotNullOrEmpty()]
        [System.String]$RosterId,

        [ValidateNotNull()]
        [Microsoft.IdentityModel.Clients.ActiveDirectory.AuthenticationResult, Microsoft.IdentityModel.Clients.ActiveDirectory, Version=5.2.8.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35] $Authentication,

        [ValidateNotNull()]
        [string] $HostName
        )
    
    if(!$HostName)
    {
        $HostName = "tasks.office.com"
    }

    if(!$Authentication)
    {
        $Authentication = Connect-AAD -HostName $HostName
    }

    $AccessToken = $Authentication.AccessTokenType.ToString() + ' ' + $Authentication.AccessToken    
    $Uri = "https://" + $HostName + "/taskAPI/v3.0/rosters('" + $RosterId + "')/members";
    $Result = Invoke-RestMethod -ContentType "application/json" -Headers @{"Accept"="application/json"; "Authorization"=$AccessToken; "Accept-Charset"="UTF-8"} -Method GET $Uri

    return $Result.value | ForEach-Object { $_.userId }
}

function Set-PlannerConfiguration
{
<#
.Synopsis
Configures tenant level settings for Microsoft Planner.
.Description
This cmdlet allows tenant administrators to change policies regarding availability of certain features in Microsoft Planner. The changes to settings take effect immediately. This cmdlet specifies the administrator preference on whether the feature should be available. The features can still be disabled due to Microsoft Planner behavior, at the discretion of Microsoft.
.Parameter HostName
Optional. The host name of the Microsoft Planner service to target. Default is tasks.office.com.
.Parameter Authentication
Optional. An AuthenticationResult obtained from the Microsoft Identity library. When this parameter is not set, the user will be prompted for credentials.
.Parameter AllowCalendarSharing
If set to $false, disables creating iCalendar links from Microsoft Planner, and disables previously created iCalendar links.  If set to $true, enables creating iCalendar links from Microsoft Planner and re-enables any previously created iCalendar links.
.Parameter AllowTenantMoveWithDataLoss
If set to $true, allows the tenant to be moved to another Planner environment or region. This move will result in the tenant's existing Planner data being lost.
.Parameter AllowRosterCreation
If set to $true, allows the users of the tenant to create rosters as the container for a plan to facilitate ad-hoc collaboration. This setting does not restrict the use of existing roster contained plans.
.Parameter AllowPlannerMobilePushNotifications
If set to $true, allows the use of direct push mobile notifications in Tenant
.example

Set-PlannerConfiguration -AllowCalendarSharing $true

.example

Set-PlannerConfiguration -AllowTenantMoveWithDataLoss $true

.example

Set-PlannerConfiguration -AllowRosterCreation $false

.example

Set-PlannerConfiguration -AllowPlannerMobilePushNotifications $false
#>
    param(
        [ValidateNotNull()]
        [string] $HostName,

        [ValidateNotNull()]
        [Microsoft.IdentityModel.Clients.ActiveDirectory.AuthenticationResult, Microsoft.IdentityModel.Clients.ActiveDirectory, Version=5.2.8.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35] $Authentication,

        [ValidateNotNullOrEmpty()]
        [Parameter(Mandatory=$false)][System.String]$AccessToken,
        [Parameter(Mandatory=$false, ValueFromPipeline=$true)][System.Boolean]$AllowCalendarSharing,
        [Parameter(Mandatory=$false, ValueFromPipeline=$true)][System.Boolean]$AllowTenantMoveWithDataLoss,
        [Parameter(Mandatory=$false, ValueFromPipeline=$true)][System.Boolean]$AllowRosterCreation,
        [Parameter(Mandatory=$false, ValueFromPipeline=$true)][System.Boolean]$AllowPlannerMobilePushNotifications
    )
    
    if(!$HostName)
    {
        $HostName = "tasks.office.com"
    }

    if(!$Authentication)
    {
        $Authentication = Connect-AAD -HostName $HostName
    }

    $Uri = "https://" + $HostName + "/taskAPI/tenantAdminSettings/Settings";
    
    if(!($PSBoundParameters.ContainsKey("AccessToken"))){
        $authorizationContext = Connect-AAD
        $AccessToken = $authorizationContext.AccessTokenType.ToString() + ' ' +$authorizationContext.AccessToken
    }
    
    $flags = @{}
    
    if($PSBoundParameters.ContainsKey("AllowCalendarSharing")){
        $flags.Add("allowCalendarSharing", $AllowCalendarSharing);
    }

    if($PSBoundParameters.ContainsKey("AllowTenantMoveWithDataLoss")){
        $flags.Add("allowTenantMoveWithDataLoss", $AllowTenantMoveWithDataLoss);
    }

    if($PSBoundParameters.ContainsKey("AllowRosterCreation")){
        $flags.Add("allowRosterCreation", $AllowRosterCreation);
    }

    if($PSBoundParameters.ContainsKey("AllowPlannerMobilePushNotifications")){
        $flags.Add("allowPlannerMobilePushNotifications", $AllowPlannerMobilePushNotifications)
    }

    $propertyCount = $flags | Select-Object -ExpandProperty Count
    
    if($propertyCount -eq 0) {
        Throw "No properties were set."
    }
    
    $requestBody = $flags | ConvertTo-Json

    Invoke-RestMethod -ContentType "application/json;odata.metadata=full" -Headers @{"Accept"="application/json"; "Authorization"=$AccessToken; "Accept-Charset"="UTF-8"; "OData-Version"="4.0;NetFx"; "OData-MaxVersion"="4.0;NetFx"} -Method PATCH -Body $requestBody $Uri
}

function Get-PlannerConfiguration
{
<#
.Synopsis
Retrieves tenant level settings for Microsoft Planner.
.Description
This cmdlet allows users and tenant administrators to retrieve policy preferences set by the tenant administrator regarding availability of certain features in Microsoft Planner.  While a feature may be permitted by a tenant administrator's preference, features can still be disabled due to Microsoft Planner behavior, at the discretion of Microsoft.
.Parameter HostName
Optional. The host name of the Microsoft Planner service to target. Default is tasks.office.com.
.Parameter Authentication
Optional. An AuthenticationResult obtained from the Microsoft Identity library. When this parameter is not set, the user will be prompted for credentials.

.example

Get-PlannerConfiguration
#>
    param(
        [ValidateNotNull()]
        [string] $HostName,

        [ValidateNotNull()]
        [Microsoft.IdentityModel.Clients.ActiveDirectory.AuthenticationResult, Microsoft.IdentityModel.Clients.ActiveDirectory, Version=5.2.8.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35] $Authentication
        )

    if(!$HostName)
    {
        $HostName = "tasks.office.com"
    }

    if(!$Authentication)
    {
        $Authentication = Connect-AAD -HostName $HostName
    }

    $Uri = "https://" + $HostName + "/taskAPI/tenantAdminSettings/Settings";
    
    if(!($PSBoundParameters.ContainsKey("AccessToken"))){
        $authorizationContext = Connect-AAD
        $accessToken = $authorizationContext.AccessTokenType.ToString() + ' ' +$authorizationContext.AccessToken
    }
    
    $response = Invoke-RestMethod -ContentType "application/json;odata.metadata=full" -Headers @{"Accept"="application/json"; "Authorization"=$AccessToken; "Accept-Charset"="UTF-8"; "OData-Version"="4.0;NetFx"; "OData-MaxVersion"="4.0;NetFx"} -Method GET $Uri
    $result = New-Object PSObject -Property @{
        "AllowCalendarSharing" = $response.allowCalendarSharing
        "AllowTenantMoveWithDataLoss" = $response.allowTenantMoveWithDataLoss
        "AllowRosterCreation" = $response.allowRosterCreation
        "AllowPlannerMobilePushNotifications" = $response.allowPlannerMobilePushNotifications
    }

    return $result
}

Export-ModuleMember -Function `
    Add-PlannerRosterMember,`
    Export-PlannerUserContent,`
    Get-PlannerConfiguration,`
    Get-PlannerRosterMember, `
    Get-PlannerUserContentExport,`
    Get-PlannerUserPolicy,`
    Remove-PlannerRosterMember,`
    Set-PlannerConfiguration, `
    Set-PlannerUserPolicy,`
    Start-PlannerUserContentExport